// Copyright (c) Tim Gromeyer
// Licensed under the MIT License - https://opensource.org/licenses/MIT

#ifndef TABLE_H
#define TABLE_H

#include <string>

[[nodiscard]] std::string formatMarkdownTable(const std::string &inputTable);

#endif // TABLE_H
